@@ -51,12 +51,17 @@ static void   toc_uri_selected_cb           (YelpHtml         *html,
 					     YelpURI          *uri,
 					     gboolean          handled,
 					     YelpViewTOC      *view);
+static void   toc_html_title_changed_cb     (YelpHtml         *html,
+					     const gchar      *title,
+					     YelpViewTOC      *view);
 static void   toc_page_start                (YelpViewTOC      *view,
-					     const gchar      *page_name);
+					     const gchar      *title,
+					     const gchar      *heading);
 static void   toc_page_end                  (YelpViewTOC      *view);
 
 enum {
 	URI_SELECTED,
+	TITLE_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -117,8 +122,12 @@ toc_init (YelpViewTOC *view)
 	priv->html_view   = yelp_html_new ();
 	priv->html_widget = yelp_html_get_widget (priv->html_view);
 
-	g_signal_connect (G_OBJECT (priv->html_view), "uri_selected",
-			  G_CALLBACK (toc_uri_selected_cb), 
+	g_signal_connect (priv->html_view, "uri_selected",
+			  G_CALLBACK (toc_uri_selected_cb),
+			  view);
+
+	g_signal_connect (priv->html_view, "title_changed",
+			  G_CALLBACK (toc_html_title_changed_cb),
 			  view);
 }
 
@@ -135,6 +144,17 @@ toc_class_init (YelpViewTOCClass *klass)
 			      yelp_marshal_VOID__POINTER_BOOLEAN,
 			      G_TYPE_NONE,
 			      2, G_TYPE_POINTER, G_TYPE_BOOLEAN);
+
+	signals[TITLE_CHANGED] = 
+		g_signal_new ("title_changed",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (YelpViewTOCClass,
+					       title_changed),
+			      NULL, NULL,
+			      yelp_marshal_VOID__STRING,
+			      G_TYPE_NONE,
+			      1, G_TYPE_STRING);
 }
 
 #if 0
@@ -176,7 +196,7 @@ toc_start (YelpViewTOC *view)
 	
  	yelp_html_clear (priv->html_view);
 
-	toc_page_start (view, page_title);
+	toc_page_start (view, page_title, page_title);
 
 	sections = priv->important_sections;
 	
@@ -315,7 +335,7 @@ toc_start (YelpViewTOC *view)
 	}
 
 	yelp_html_printf (priv->html_view,
-			  "</ul></td><td bgcolor=\"#3b3b4f\" width=\"10%\">&nbsp;</td></tr>");
+			  "</ul></td><td bgcolor=\"#3b3b4f\" width=\"10%%\">&nbsp;</td></tr>");
 	
 	toc_page_end (view);
 
@@ -358,6 +378,7 @@ toc_man_emit (YelpViewTOC *view, GNode *first)
 	YelpSection     *section;
 	gboolean         got_a_leaf;
 	char            *path, *url;
+	char            *tmp;
 	int              i;
 	gboolean         sub_started = FALSE;
 	gchar           *str_docs = _("Documents");
@@ -397,7 +418,7 @@ toc_man_emit (YelpViewTOC *view, GNode *first)
 		yelp_html_printf (priv->html_view, "<h3>%s</h3>", str_docs);
 
 		yelp_html_write (priv->html_view,
-				 "<table cellpadding=\"2\" cellspacing=\"2\" border=\"0\" width=\"100%\">\n",
+				 "<table cellpadding=\"2\" cellspacing=\"2\" border=\"0\" width=\"100%%\">\n",
 				 -1);
 
 		i = 0;
@@ -420,9 +441,11 @@ toc_man_emit (YelpViewTOC *view, GNode *first)
 			
 				section = node->data;
 				url = yelp_util_compose_path_url (node->parent, yelp_uri_get_path (section->uri));
-				yelp_html_printf (priv->html_view, "<td valign=\"Top\"><a href=\"%s\">%s</a></td>\n", yelp_uri_to_string (section->uri), 
+				tmp = yelp_uri_to_string (section->uri);
+				yelp_html_printf (priv->html_view, "<td valign=\"Top\"><a href=\"%s\">%s</a></td>\n", tmp, 
 						  section->name);
 				g_free (url);
+				g_free (tmp);
 				i++;
 			}
 		} while ((node = node->next) != NULL);
@@ -441,6 +464,7 @@ toc_man_2 (YelpViewTOC *view,
 	GNode           *first;
 	gchar           *name;
 	gchar           *string = _("Manual pages");
+	gchar           *title;
 
 	g_return_if_fail (YELP_IS_VIEW_TOC (view));
 	
@@ -453,21 +477,24 @@ toc_man_2 (YelpViewTOC *view,
 	first = root->children;
 
 	yelp_html_clear (priv->html_view);
-
-	toc_page_start (view, string);
-
+ 
 	name = toc_full_path_name (view, root);
+	
+	title = g_strdup_printf ("%s/%s", string, name);
 
-	yelp_html_printf (priv->html_view,
-			  "<td colspan=\"2\"><h2>%s/%s</h2><ul>",
-			  string, name);
+	toc_page_start (view, title, string);
 
+ 	yelp_html_printf (priv->html_view,
+ 			  "<td colspan=\"2\"><h2>%s</h2><ul>",
+ 			  title);
+
+	g_free (title);
 	g_free (name);
 
 	toc_man_emit (view, first);
 
 	yelp_html_printf (priv->html_view,
-			  "</td><td bgcolor=\"#3b3b4f\" width=\"10%\">&nbsp;</td></tr>\n");
+			  "</td><td bgcolor=\"#3b3b4f\" width=\"10%%\">&nbsp;</td></tr>\n");
 		
 	toc_page_end (view);
 
@@ -502,7 +529,7 @@ toc_man_1 (YelpViewTOC *view)
 
 	yelp_html_clear (priv->html_view);
 
-	toc_page_start (view, string);
+	toc_page_start (view, string, string);
 
 	yelp_html_printf (priv->html_view,
 			  "<td colspan=\"2\"><h2>%s</h2><ul>",
@@ -524,7 +551,7 @@ toc_man_1 (YelpViewTOC *view)
 	} while ((node = g_node_next_sibling (node)));
 
 	yelp_html_printf (priv->html_view,
-			  "</ul></td><td bgcolor=\"#3b3b4f\" width=\"10%\">&nbsp;</td></tr>");
+			  "</ul></td><td bgcolor=\"#3b3b4f\" width=\"10%%\">&nbsp;</td></tr>");
 
 	toc_page_end (view);
 
@@ -558,7 +585,7 @@ toc_info (YelpViewTOC *view)
 
 	yelp_html_clear (priv->html_view);
 
-	toc_page_start (view, string);
+	toc_page_start (view, string, string);
 
 	yelp_html_printf (priv->html_view, 
 			  "<td colspan=\"2\"><h2>%s</h2><ul>", str_docs);
@@ -578,7 +605,7 @@ toc_info (YelpViewTOC *view)
 	} while ((node = g_node_next_sibling (node)));
 	
 	yelp_html_printf (priv->html_view,
-			  "</ul></td><td bgcolor=\"#3b3b4f\" width=\"10%\">&nbsp;</td></tr>");
+			  "</ul></td><td bgcolor=\"#3b3b4f\" width=\"10%%\">&nbsp;</td></tr>");
 
 	toc_page_end (view);
 
@@ -654,12 +681,21 @@ toc_uri_selected_cb (YelpHtml    *html,
 }
 
 static void
-toc_page_start (YelpViewTOC *view, const gchar *page_name)
+toc_html_title_changed_cb (YelpHtml    *html, 
+			   const gchar *title,
+			   YelpViewTOC *view)
+{
+	g_signal_emit (view, signals[TITLE_CHANGED], 0, title);
+}
+
+static void
+toc_page_start (YelpViewTOC *view, const gchar *title, const gchar *heading)
 {
 	YelpViewTOCPriv *priv;
 	
 	g_return_if_fail (YELP_IS_VIEW_TOC (view));
-	g_return_if_fail (page_name != NULL);
+	g_return_if_fail (title != NULL);
+	g_return_if_fail (heading != NULL);
 	
 	priv = view->priv;
 	
@@ -667,37 +703,34 @@ toc_page_start (YelpViewTOC *view, const gchar *page_name)
 			  "<html>\n"
 			  "<head>\n"
 			  "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n"
-			  "<style type=\"text/css\">\n"
-			  "A:link { color: #00008b; font-size: 10 }\n"
-			  "A:visited { color: #00008b; font-size: 10}\n"
-			  "A:active { color: #00008b; font-size: 10}\n"
-			  "h1 { font-size: 25 }\n"
-			  "h2 { font-size: 15 }\n"
-			  "BODY { color: #000000; font-size: 10 }\n"
-			  "</style>\n"
+			  "<title>%s</title>\n"
 			  "</head>\n"
-			  "<body bgcolor=\"white\" text=\"black\" marginwidth=\"0\"\n"
+			  "<body marginwidth=\"0\"\n"
 			  "background=\"file:" IMAGEDIR "/bcg.png\" marginheight=\"0\"\n"
 			  "border=\"10\" topmargin=\"0\" leftmargin=\"0\">\n"
-			  "<table width=\"100%\" border=\"0\" cellpadding=\"4\" cellspacing=\"0\">\n"
+			  "<table width=\"100%%\" border=\"0\" cellpadding=\"4\" cellspacing=\"0\">\n"
 			  "<tr valign=\"top\">\n"
 			  "<td width=\"75\">\n"
 			  "<img alt=\"\" src=\"file:" IMAGEDIR "/empty.png\" width=\"75\" height=\"1\">\n"
 			  "</td>\n"
 			  "<td colspan=\"2\"><h1>%s</h1></td>\n"
-			  "<td bgcolor=\"#3b3b4f\" width=\"10%\">&nbsp;</td>\n"
+			  "<td bgcolor=\"#3b3b4f\" width=\"10%%\">&nbsp;</td>\n"
 			  "</tr>\n"
 			  "<tr valign=\"top\">\n"
 			  "<td width=\"75\">\n"
 			  "<img alt=\"\" src=\"file:" IMAGEDIR "/empty.png\" width=\"75\" height=\"1\">\n"
 			  "</td>\n",
-			  page_name);
+			  title,
+			  heading);
 }
 
 static void
 toc_page_end (YelpViewTOC *view)
 {
 	YelpViewTOCPriv *priv;
+	gchar           *str_gnome_is  = _("GNOME is");
+	gchar           *str_fs        = _("Free Software");
+	gchar           *str_copyright = _("(C)");
 	
 	g_return_if_fail (YELP_IS_VIEW_TOC (view));
 	
@@ -711,21 +744,24 @@ toc_page_end (YelpViewTOC *view)
 			  "<td colspan=\"2\">\n"
 			  "<img src=\"file:" IMAGEDIR "/empty.png\" height=\"200\">\n"
 			  "</td>\n"
-			  "<td bgcolor=\"#3b3b4f\" width=\"10%\">&nbsp;</td></tr>\n"
+			  "<td bgcolor=\"#3b3b4f\" width=\"10%%\">&nbsp;</td></tr>\n"
 			  "<tr valign=\"top\">\n"
 			  "<td width=\"75\"><img alt=\"\" src=\"file:" IMAGEDIR "/empty.png\" width=\"75\" height=\"1\"></td>\n"
 			  "<td>\n"
-			  "<a href=\"http://www.gnome.org\" target=\"_blank\">\n"
-			  "<img src=\"file:" IMAGEDIR "/gnome2-button.png\" alt=\"(c) 2002 Gnome hackers\" width=\"64\" height=\"64\" border=\"0\">\n"
+			  "<a href=\"http://www.gnome.org\" target=\"_blank\">"
+			  "<img src=\"file:" IMAGEDIR "/gnome2-button.png\" border=\"0\">\n"
 			  "</a>\n"
 			  "</td>\n"
 			  "<td>\n"
-			  "(C) 1998-2002 Gnome Hackers<br>\n"
-			  "Gnome is <a href=\"http://www.gnu.org/philosophy/free-sw.html\">Free Software</a>. |<a href=\"http://www.gnome.org\">http://www.gnome.org</a>\n"
+			  "<font size=\"-1\">%s <a href=\"http://www.gnu.org/philosophy/free-sw.html\">%s</a> | <a href=\"http://www.gnome.org\">http://www.gnome.org</a><br>\n"
+			  "%s 2001-2002 Mikael Hallendal</font>\n"
 			  "</td>\n"
-			  "<td bgcolor=\"#3b3b4f\" width=\"10%\">&nbsp;</td>\n"
+			  "<td bgcolor=\"#3b3b4f\" width=\"10%%\">&nbsp;</td>\n"
 			  "</tr>\n"
-			  "</table></body></html>\n");
+			  "</table></body></html>\n",
+			  str_gnome_is,
+			  str_fs,
+			  str_copyright);
 }
 
 YelpViewTOC *
@@ -766,7 +802,7 @@ toc_scrollkeeper (YelpViewTOC *view, GNode *root)
 
 	yelp_html_clear (priv->html_view);
 
-	toc_page_start (view, name);
+	toc_page_start (view, name, name);
 
 	yelp_html_printf (priv->html_view, "<td colspan=\"2\">");
 
@@ -829,7 +865,7 @@ toc_scrollkeeper (YelpViewTOC *view, GNode *root)
 	}
 
 	yelp_html_printf (priv->html_view,
-			  "</ul></td><td bgcolor=\"#3b3b4f\" width=\"10%\">&nbsp;</td></tr>");
+			  "</ul></td><td bgcolor=\"#3b3b4f\" width=\"10%%\">&nbsp;</td></tr>");
 	
 	toc_page_end (view);
 
